/*
	ChangeWaveFileCmd.cc

	Will Eighteen, 2000: will@opaline.demon.co.uk             
*/

#include "ChangeWaveFileCmd.h"
#include "OpResources.h"
#include "AppWindow.h"	/* access to wavePak struct */


enum OPTYPE { REMOVE, INSERT };


ChangeWaveFileCmd::ChangeWaveFileCmd ( char *name, int active, int source, int target, int opType ) : BreakCmd ( name, active )
{
	if ( target == 0 )
		return;

	if ( opType > INSERT )
		return;

	char baseName[BUFSIZE];
	char revNum[12];
	int error;
	unsigned int baseLength;
	FILE *file;

	strncpy ( _dstFile, (*wavePak)[target-1].pWave->name(), sizeof ( _dstFile ) );
	_dstFile [ sizeof ( _dstFile ) - 1 ] = 0;
	baseLength = strstr ( _dstFile, ".wav" ) - _dstFile;
	
	if ( baseLength > ( sizeof ( baseName ) - 1 ) )
		baseLength = sizeof ( baseName ) - 1;

	strncpy ( baseName, _dstFile, baseLength );
	baseName [ baseLength ] = 0;
	(*wavePak)[target-1].pWave->filePos = 0;

	if ( opType == INSERT )
	{
		if ( source == 0 )
			return;	// no source to be copied (or space to insert)
		else
			_srcWaveNum = source;
		
		strncpy ( _srcFile, (*wavePak)[source-1].pWave->name(), sizeof ( _srcFile ) );
		(*wavePak)[source-1].pWave->filePos = (*wavePak)[source-1].startSel<<1;
	}
	else
		strncpy ( _srcFile, _dstFile, sizeof ( _srcFile ) );

	_srcFile [ sizeof ( _srcFile ) - 1 ] = 0;
	_dstWaveNum = target;
	error = 1;
	
	// set up  a temporary output file for the operation	
	while ( error == 1 )	// file exists
	{
		strncpy ( _tmpFile, baseName, sizeof ( _tmpFile ) );
		strncat ( _tmpFile, "-", sizeof ( _tmpFile ) - strlen ( _tmpFile ) - 1 );
		snprintf ( revNum, 12, "%u", (*wavePak)[target-1].revision );
		strncat ( _tmpFile, revNum, sizeof ( _tmpFile ) - strlen ( _tmpFile ) - 1 );
		strncat ( _tmpFile,".wav", sizeof ( _tmpFile ) - strlen ( _tmpFile ) - 1 );
		_tmpFile [ sizeof ( _tmpFile ) - 1 ] = 0;

		if ( ( file = fopen ( _tmpFile, "r" ) ) == NULL )
			error = 0;	// file doesn't exist, name is unique
		else
		{
			fclose ( file );
			(*wavePak)[target-1].revision++;
			error = 1;
		}
	}

	_opType = opType;

	switch ( _opType )
	{
		case REMOVE:	// 'cut selection'
			_numBytes = (*wavePak)[target-1].pWave->dataLength() - ( ( (*wavePak)[target-1].endSel - (*wavePak)[target-1].startSel + 1 )<<1 );
			break;
		
		case INSERT:	// 'insert selection'
			_numBytes = (*wavePak)[target-1].pWave->dataLength() + ( ( (*wavePak)[source-1].endSel - (*wavePak)[source-1].startSel + 1 )<<1 );
			break;

		default:
			_numBytes = 0;
	}	
	
	_pSave = new Wave;    // new instance for (processed) wave
	_pSave->FillHeader( (*wavePak)[target-1].pWave->numChannels(),
						(*wavePak)[target-1].pWave->sampleRate() ,
						(*wavePak)[target-1].pWave->bitsPerSample(), 0 );	// no samples yet
	_pSave->SaveHeader( _tmpFile );
	_pSave->filePos = 0;
	_percentDone = 0;

	// initial timeout empirically determined for particular type of operation
	updateTimeout ( mixwaveResources->CWF_callbacktimeout() );
}


ChangeWaveFileCmd::~ChangeWaveFileCmd()
{	
	// empty
}
	

void ChangeWaveFileCmd::doCommand()
{
	long blockLength;

	if ( _opType == INSERT )
	{
		if ( (*wavePak)[_srcWaveNum-1].fLock == ACTIVE )
		{
			delete _pSave;
			return;
		}
	}

	if ( (*wavePak)[_dstWaveNum-1].fLock == ACTIVE )
	{
		delete _pSave;
		return;
	}

	if ( (*wavePak)[_dstWaveNum-1].filePerm == 1 )
	{
		delete _pSave;
		return;	// destination file is read-only
	}
	
	blockLength = mixwaveResources->blockLength();

	if ( _pSave->filePos < ((*wavePak)[_dstWaveNum-1].startSel<<1 ) )
	{	// move first part of dst file to tmp
		if ( blockLength > ( (*wavePak)[_dstWaveNum-1].startSel<<1 ) - _pSave->filePos )
			blockLength = ( (*wavePak)[_dstWaveNum-1].startSel<<1 ) - _pSave->filePos;

		(*wavePak)[_dstWaveNum-1].pWave->LoadBlock ( _pSave->filePos, blockLength );
		(*wavePak)[_dstWaveNum-1].pWave->AppendBlock ( _tmpFile,
								(*wavePak)[_dstWaveNum-1].pWave->numChannels(),
								(*wavePak)[_dstWaveNum-1].pWave->sampleRate(),
								(*wavePak)[_dstWaveNum-1].pWave->bitsPerSample(),
								blockLength>>1 );

		(*wavePak)[_dstWaveNum-1].pWave->filePos += blockLength;
		writeHeader ( blockLength );
		update();
		return;
	}

	if ( ( (*wavePak)[_dstWaveNum-1].pWave->filePos>>1 < ( (*wavePak)[_dstWaveNum-1].endSel + 1 ) ) && ( _opType == REMOVE ) )
		(*wavePak)[_dstWaveNum-1].pWave->filePos = ( (*wavePak)[_dstWaveNum-1].endSel + 1 )<<1;
	else
	{	// insert _selected_ samples, inclusive

		if ( _opType == INSERT )
		{
			if ( (*wavePak)[_srcWaveNum-1].pWave->filePos>>1 != (*wavePak)[_srcWaveNum-1].endSel + 1 )
			{
				blockLength = mixwaveResources->blockLength();

				if ( blockLength > ( ( ( (*wavePak)[_srcWaveNum-1].endSel + 1 )<<1 ) -
									(*wavePak)[_srcWaveNum-1].pWave->filePos ) );
					blockLength = ( ( ( (*wavePak)[_srcWaveNum-1].endSel + 1)<<1 ) -
										(*wavePak)[_srcWaveNum-1].pWave->filePos );

				(*wavePak)[_srcWaveNum-1].pWave->LoadBlock ( (*wavePak)[_srcWaveNum-1].pWave->filePos, blockLength );
				(*wavePak)[_srcWaveNum-1].pWave->AppendBlock ( _tmpFile,
										(*wavePak)[_dstWaveNum-1].pWave->numChannels(),
										(*wavePak)[_dstWaveNum-1].pWave->sampleRate(),
										(*wavePak)[_dstWaveNum-1].pWave->bitsPerSample(),
										blockLength>>1 );
									
				(*wavePak)[_srcWaveNum-1].pWave->filePos += blockLength;
				writeHeader ( blockLength );
				update();
				return;
			}
		}
	}

	if ( (*wavePak)[_dstWaveNum-1].pWave->filePos < (*wavePak)[_dstWaveNum-1].pWave->dataLength() )
	{	// copy target after modification point
		blockLength = mixwaveResources->blockLength();

		if ( blockLength > ( (*wavePak)[_dstWaveNum-1].pWave->dataLength() - (*wavePak)[_dstWaveNum-1].pWave->filePos ) )
			blockLength = (*wavePak)[_dstWaveNum-1].pWave->dataLength() - (*wavePak)[_dstWaveNum-1].pWave->filePos;

		(*wavePak)[_dstWaveNum-1].pWave->LoadBlock ( (*wavePak)[_dstWaveNum-1].pWave->filePos, blockLength );
		(*wavePak)[_dstWaveNum-1].pWave->AppendBlock ( _tmpFile,
									(*wavePak)[_dstWaveNum-1].pWave->numChannels(),
									(*wavePak)[_dstWaveNum-1].pWave->sampleRate(),
									(*wavePak)[_dstWaveNum-1].pWave->bitsPerSample(),
									blockLength>>1 );									
	
		(*wavePak)[_dstWaveNum-1].pWave->filePos += blockLength;
		writeHeader ( blockLength );
		update();
	}

	if ( _pSave->filePos >= _numBytes )	// all data transferred to target file
	{	// eq if cut, gt if copy (cut ignores start selection sample)
		_done = True;
		delete _pSave;
	}
}


void ChangeWaveFileCmd::writeHeader ( long blockLength )
{
	_pSave->filePos += blockLength;
	_pSave->FillHeader ( _pSave->numChannels(), _pSave->sampleRate(),
						_pSave->bitsPerSample(), _pSave->filePos>>1 );
	_pSave->ChangeHeader ( _tmpFile );
}


void ChangeWaveFileCmd::update()
{
	char buf[5];
	int percent;

	percent = ( int ) ( ( float ) _pSave->filePos / ( float ) _numBytes * 100 );

	if ( _percentDone != percent )
	{
		_percentDone = percent;
		snprintf ( buf, 5, "%d%%", _percentDone );
		updateMessage ( buf );
	}
}
