/*
	MixBufferSizeCmd.cc

	Will Eighteen, 2000: will@opaline.demon.co.uk
*/

#include "MixBufferSizeCmd.h"
#include "Application.h"
#include <LineEdit.h>
#include <MessageDialogManager.h>

#include "OpResources.h"	// this is the only resources file with global resources

MixBufferSizeCmd::MixBufferSizeCmd ( const char *name, int active ) :  Cmd ( name, active )
{
	_popupRealised = False;
}


void MixBufferSizeCmd::doCommand()
{
	if ( _popupRealised == True )
		return;

	char buf[30];
	char help[100];
	
	_edit = new LineEdit ( ( char * ) name(), TRUE,
									&MixBufferSizeCmd::enterCallback,
									&MixBufferSizeCmd::cancelCallback,
									( void * ) this );	
                
	/* retrieve resource to be displayed */
 	snprintf ( buf, sizeof ( buf ),"%ld", mixwaveResources->blockLength() );
	buf [ sizeof ( buf ) - 1 ] = 0;
	_edit->setColumns ( strlen ( buf ) );	// set width before instantiation
	snprintf ( help, sizeof ( help ), "Specifies size of buffer used per wave during mixdown: bytes;\
default %ld", mixwaveResources->blockLength() );
	help [ sizeof ( help ) - 1 ] = 0;
	_edit->setHelp ( help );
	_edit->execute();
	_edit->setText ( buf );	// can't set text before instantiation...
	_popupRealised = True;
}


void MixBufferSizeCmd::undoCommand()
{
	// not implemented
}


void MixBufferSizeCmd::enterCallback ( void *clientData, char *text )
{
	int num;

	MixBufferSizeCmd *obj = ( MixBufferSizeCmd * ) clientData;
	sscanf ( text, "%d", &num );	

	if ( num >= 0 )	// we allow 0 so as to be able to force 'load entire wave'
		obj->enter ( num );
	
	obj->_popupRealised = False;
	delete obj->_edit;
}


/* specify resource to be set on 'enter' or 'ok' */
void MixBufferSizeCmd::enter ( int num )
{
    // allocated blockLength _must_ be an integer multiple of
    // bytesPerSample
	int temp = num;
	int bytesPerSample = mixwaveResources->bitsPerSample() / 8;
	float fval = ( float ) num / ( float ) bytesPerSample;
	int ival = ( int ) ( num / bytesPerSample );

	while ( ( fval - ( float ) ival ) != 0 )
	{
		num++;
		fval = ( float ) num / ( float ) bytesPerSample;;
		ival = ( int ) ( num / bytesPerSample );
	}
	
	if ( num != temp )
	{
		if ( mixwaveResources->enableWarningPopups() == True )
		{
			char msg[512];
			snprintf ( msg, sizeof ( msg ), "Size specified not an integer \
multiple\nof bytesPerSample ( %d ). Size set to\n%d.", bytesPerSample, num );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}
	}

	mixwaveResources->blockLength ( num );
}


void MixBufferSizeCmd::cancelCallback ( void *clientData )
{
	MixBufferSizeCmd *obj = ( MixBufferSizeCmd * ) clientData;

	obj->_popupRealised = False;
	delete obj->_edit;
}
