/*
    SpatialHelpCmd.cc

    Will Eighteen, 2006: will@opaline.demon.co.uk
*/

#include "SpatialHelpCmd.h"
#include "Application.h"
#include <TextEdit.h>


SpatialHelpCmd::SpatialHelpCmd ( const char *name, int active ) :  NoUndoCmd ( name, active )
{
	_editRealised = FALSE;
}


SpatialHelpCmd::~SpatialHelpCmd()
{
	// empty
}


void SpatialHelpCmd::doCommand()
{
	if (_editRealised == TRUE )
		return;

	_edit = new TextEdit ( ( char * ) name(), TRUE, &SpatialHelpCmd::okCallback, &SpatialHelpCmd::cancelCallback, ( void * ) this );	
	_edit->setRows ( 19 );
	_edit->setColumns ( 70 );
	_editRealised = TRUE;
	_edit->execute();
	_edit->setText ( "New / Template / Spatial\nPops up a menu of loaded \
wave files. Left-clicking a button causes the spatialisation control system \
to be activated for the selected wave. This is in effect an advanced pan.\
\n\nIn place of simply adjusting the L/R signal balance, it incorporates reflections \
from objects left or right from the sound source, absorption by a transmission \
medium, differential path length from the sound source to the ears (when this \
source is not centrally place with respect to the listener) and doppler effect \
(to simulate a receding sound source).\nIts aim is less the imposition of \
effects and more the accurate auditory placement of a sound. In this respect it \
is experimental; the physical model employed is simplistic; the operation \
of this section of mixwave largely untested. It doubtless has faults.\n\
Inappropriate choice of parameter values may lead to 'glitches' in the output \
file; the program 'csound' is utilised to render the sound file and it cannot \
cope with meaningless input, which can be the case when mixwave is used to \
supply it with information.\nThe 'Spatialise' command uses default values \
designed to avoid these problems. Although parameter values may be extended \
beyond the defaults given, care should be taken to consult the csound manual \
(freely available on the net as is the csound program itself) for the \
limitations on use of its functions.\n\nActivating the 'Spatialise' button \
causes a variety of popups to appear:\n\n1) Wave viewer;\n2) Data display;\n\
3) Sound source parameters;\n4) Reflection \
parameters;\n5) Range parameters.\n6) Template function plotter;\n\n\n1) Wave \
Viewer.\nThe wave file is loaded read-only; only \
the template files controlling the spatialisation process can be edited. If a \
selection is made in the view pane, it is reflected in any open template view \
popups. The right-hand pane command buttons determine the view pane contents \
of the dependent template view panes, e.g. 'zoom in' causes the selected \
region to be displayed in the template view.\n\n\n2) Data display.\nThis ia \
also cross-coupled to the wave and template views. Data edits using the \
'%peak' data field, as for the wave view 'magnitude' data field, will be \
ignored (although they make a temporary difference to the view).\n\n\n3) Sound \
source parameters.\nThis panel of buttons controls the pitch and quality of \
the object presumed to be emitting the sound. Button names including 'envelope' \
cause an appropriate template view to pop up. Note that there is no 'close' \
facility for template views: once activated, they remain on-screen until the \
wave view is closed with 'Ok' or 'Cancel'. A realised template view may however \
be minimised by double left-clicking on the title bar or iconified with the \
window manager 'iconify' button.\nTemplate views are writeable: any shape may \
be drawn in a template view pane with the mouse (right-click and drag) or by \
use of the 'ShapeSpatialTemplate' popup (the template function plotter). Note \
that in using the latter it is necessary to select the appropriate template \
as the target for its operations. This is only possible when the relevant \
template view is realised and a region selected in it; if these conditions \
are satisfied, then clicking the template name in the menus will cause the \
name of that template to appear at the top of the ShapeSpatialTemplate popup \
above the command buttons. Hand-drawn templates may be smoothed by using \
'All' and 'Sel range' in the wave view, then 'Smooth' in the template function \
plotter (or by selecting the same in the template view itself).\n\
Note that before drawing in the template view panes it is often useful to \
ensure the whole wave is viewed (and thus also the whole associated template) \
by initially clicking 'All' in the wave view popup.\n\na) reverb envelope\n\
A drawable window allowing reverb values in the range [0,1]. These constitute \
a 'gain factor' controlling the 'reverb send', i.e. the amount of dry signal \
passed to the csound reverb opcode 'reson' (which is controlled by this \
template). Larger values mean a greater amount of signal is sent to the \
reverb unit, hence it is a larger component of the output (which is combined \
dry and reverb signals) and so sounds more prominent.\n\nb) distance envelope\
\nAllows specification of a gain factor; as the factor increases, representing \
increasing distance, the output level is reduced. A factor of 1 is the default, \
indicating no attenuation. The allowed range is [1,10]. It should be noted \
that this also has an effect on the amount of doppler effect, if this is \
used.\n\nc) bandwidth envelope\nControls a filter applied to the sound source \
to simulate absorption by a transmission medium; default range [0,11025] \
Hz.\n\nd) sweep freq. envelope\nControls the centre frequency of the filter \
used in c) above, allowing selection of a specific range of frequencies to \
be attenuated; default range [0,11025].\n\ne) pan envelope\nDetermines \
the position of the sound \
between the left and right apeakers; range [0,1].\n\nf) doppler pitch \
envelope\nThis is used to simulate the effect of a receding sound source, e.g. \
a motor vehicle. The default value of unity provides for an unchanged signal; \
as the value is reduced, the pitch shift applied becomes more pronounced.\n\n\
g) pan function\nPops up a data entry box with default value 0. This is a \
constant-power pan: traversing the range of pan values from 0 to 1 results in \
the sound appearing to move in horizontal fashion across the auditory field \
with no apparent change of distance (unless other parameters have been \
specified). Other valid values are 1 and 2. See Mixwave/doc/commented_\
spatial.orc for what these actually do (lines 108-140).\nNote that in all \
data entry popups, 'Return' must be used to confirm a new value or the old \
value will be preserved.\n\nh) filter type\n\
This data entry box allows change in the type of filter: both band (0) and \
notch (1) are supported.\n\ni) angular separation\nCompensates for the type of \
sound reproduction used. See the 'Help' button in the popup. Some \
experimentation may be necessary to achieve the best results.\n\nj) scale \
factor\nThis is a global gain value; use of multiple effects may cause the \
output level to drop considerably. Gain values >1 compensate for this.\n\nk) \
detune value\nThis applies two copies of the input wave file, detuned in \
accordance with the control value entered into the popup. It has no effect \
unless both the doppler envelope contains values less than unity and the \
phaser status is set to 1 (on). This effect is intended to simulate the \
'phasing' effect of a moving sound source.\n\nl) phaser status\nThis controls \
whether the doppler-affected signal is allowed to be detuned. If the phaser \
status is '1' (on) then two copies of the pitch-changed doppler-affected signal \
are added into the 'dry' original signal. One is slightly higher, the other \
slightly lower in frequency than the original sound. See Mixwave/doc/commented\
_spatial.orc lines 64-74.\n\nm) reverb tail\nSince reverb is a train of echoes \
of very short duration, the resulting modified sound is slightly longer than \
the original, This allows the output wave file to be lengthened to accomodate \
the the extra length caused by the reverb. The value is a time in seconds; \
typically this will need to be fairly short (but dependent on the amount of \
reverb: a larger reverb will take longer to die away). The default is 0.\n\nn) \
Quit\nQuits the 'Sound source parameters' popup. Note that this can only be \
re-invoked by closing the wave view and re-opening with 'New / Template / \
Spatial'.and then selecting the wave again. Clicking 'Ok' when the message \
'Spatial template for <name>.wav already exists: continue?' appears will \
cause the previously created templates to be re-loaded.\n\n\n4) Reflection \
parameters.\nThese control the contribution from reflective surfaces or \
objects. The left and right fields with respect to the sound source may be \
controlled independently.\n\na) <L/R> rvb level envelope\nAllows a reverb to be \
specified for the left and right reflections from the sound source.\n\nb) <L/R> \
pan envelope\nMotion of a reflected sound is controlled here (i.e. the \
movement of a sound-reflecting object).\n\nc) <L/R> pan function\nValues in \
the range [0,2] may be used. '0' gives a constant-power pan. See the pan \
function help in the 'Sound source parameters' popup (hint: try a different \
value in the pan function popups, press 'Return', pop up the pan function \
text box and hit 'help'; another hint: try left-click and mouse drag right \
past the end of the text box in \
the help text).\n\nd) <L/R> bw envelope\nBandwidth specification \
for the filters applied to the left and right reflections.\n\ne) <L/R> sweep \
envelope\nSpecifies the sweep frequency variation over the sound duration \
of the centre-frequency of the filter applied to the left and right \
reflections.\n\nf) <L/R> amplitude envelope\nControls variation in gain of \
the left and right reflections.\n\ng) <L/R> delay\nProvides for an echo \
effect, where the delay between origin and reflection is considerably \
greater than the successive delays given by a typical reverb. See 'Mixwave/\
doc/commented_spatial.orc lines 256-272.\n\nh) <L/R> filter type\nCan be \
either band (0) or notch (1).\n\ni) Quit\nQuits the reflection parameters \
popup. THe comments above under 'Sound source parameters' 'Quit' apply. \n\n\n\
5) Range parameters.\nThese control the range of values used by the template \
initialisation process, allowing the default values used to be changed. Note \
that a value change _must_ precede activation of the appropriate template \
view. The range of values allowed to a template cannot be altered after it \
has been popped up. If it is desired to reset the range of a particular \
template and that template view is currently activated, the only recourse is \
to quit using 'Ok' or 'Cancel as appropriate in the wave viewer, then reload \
the spatial templates with 'New / Template / Spatial' and re-selecting the \
wave file.\nNote that the range parameters affect the range of values allowed \
to both sound source and reflection parameters.\nSaving a state \
file allows selection information to be preserved for both the wave view and \
associated templates.\nTemplate selections are not \
reflected in the wave view, i.e. Options / System control / Time sync. has \
no effect here.\nData entered into a range popup should be confirmed by \
pressing 'Return'.\n\n\n6) Template function plotter.\nThis displays a variety \
of functions which may be used to obtain a predefined shape in a template \
selection region. Functions will not operate if no selection exists in a \
template view.\nFor the function plotter to be used, the template view must \
be realised and a region selected in it. Left-clicking the template view in \
either the 'Sound source parameters' or 'Reflection parameters' as appropriate \
causes the name of the selected template to appear in the ShapeSpatialTemplate \
popup (the function plotter).\nNote that the first function (Line) initial \
and final values are automatically set to the magnitude of the first \
and last samples in the selection.\nThe 'Smooth' function is used to remove \
'rough edges' caused by drawing with the mouse.");

	_edit->setEditStatus ( False );
}


void SpatialHelpCmd::okCallback ( XtPointer clientData, char * text )
{
	SpatialHelpCmd *obj = ( SpatialHelpCmd * ) clientData;

	obj->ok( clientData );	// ok mapped to 'enter'
}


void SpatialHelpCmd::cancelCallback ( XtPointer clientData )
{
	SpatialHelpCmd *obj = ( SpatialHelpCmd * ) clientData;

	obj->ok( clientData );	// call ok to delete object
}


void SpatialHelpCmd::ok ( void *clientData )
{
	SpatialHelpCmd *obj = ( SpatialHelpCmd * ) clientData;

	if (obj->_editRealised == TRUE )
	{
		obj->_editRealised = FALSE;
		delete obj->_edit;
	}
}
