/*
    WavToIntCmd.cc

    Will Eighteen, 2000: will@opaline.demon.co.uk
*/

#include "WavToIntCmd.h"
#include "OpResources.h"
#include <FileAccess.h>
#include "wave/waveclass.h"
#include <MessageDialogManager.h>


WavToIntCmd::WavToIntCmd ( char *name, int active, char *wavFile, char *intFile ) : BreakCmd ( name, active )
{
	strncpy ( _srcFile, wavFile, BUFSIZE );
	_srcFile [ sizeof ( _srcFile ) - 1 ] = 0;
	strncpy ( _dstFile, intFile, BUFSIZE );
	_dstFile [ sizeof ( _dstFile ) - 1 ] = 0;
	_once = 0;
	_blockLength = mixwaveResources->blockLength();
	_percentDone = 0;
	_error = 0;
	_offset = mixwaveResources->convertOffsetInt();
	_scale = mixwaveResources->convertScaleInt();
}


WavToIntCmd::~WavToIntCmd()
{	
	// empty
}
	

void WavToIntCmd::doCommand()
{
	if ( _done == True )
		return;

	int count, percent, shift, value;
	int error = 0;
	char text[5];
	unsigned int i, j;

	if ( !_once )
	{
		// can we open source for read?
		FileAccess *src = new FileAccess ( _srcFile, "r" );
		
		if ( src->errNum() != 0 )
		{
			char msg[BUFSIZE*2];
			
			error++;
			snprintf ( msg, sizeof ( msg ), "%s\nFile read error: %s", _srcFile, src->error() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}
		
		delete src;
		
		// can we open dst for write?
		FileAccess *dst = new FileAccess ( _dstFile, "w" );
		
		if ( dst->errNum() != 0 )
		{
			char msg[BUFSIZE*2];
			
			error++;
			snprintf ( msg, sizeof ( msg ), "%s\nmFile write error: %s", _dstFile, dst->error() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}
		
		delete dst;
		
		if ( error )
		{
			_done = True;
			_error = error;
			return;
		}

		Wave *pWave = new Wave;
		
		if ( !pWave->LoadHeader ( _srcFile ) )
		{
			char msg[BUFSIZE];

    		strncpy ( msg, pWave->name(), sizeof ( msg ) );
			msg [ sizeof ( msg ) - 1 ] = 0;
			strncat ( msg, "\nWave::LoadHeader failed.", sizeof ( msg ) - strlen ( msg ) - 1 );
			messageDialogManager->post ( msg );
			delete pWave;
			_done = True;
			_error++;
			return;
		}
		
		FileAccess *in = new FileAccess ( _srcFile, "r" );
		FileAccess *out = new FileAccess ( _dstFile, "w"  );
		
		_in = in;
		_out = out;
		_dataLength = pWave->dataLength();
		_in->filePos ( pWave->HeaderSize() );
		delete pWave;
		
		if ( _blockLength == 0 )
			_blockLength = in->fileSize() - 1;

		_out->elementSize ( sizeof ( int ) );	// 4-byte integer format
		_out->numElements ( 1 );
		updateTimeout ( ( int ) ( pWave->dataLength() / _blockLength ) + 1 );
		_once = 1;
	}
	
	if ( _blockLength > _dataLength )
		_blockLength =  _dataLength;

	char buf[_blockLength];
	shift = ( _bitsPerSample / 8 ) - 1;

	if ( _dataLength > 0 )
	{
		_in->readF ( buf, _blockLength, 1 );

		for ( count = 0; count < _blockLength>>shift; count++ )
		{
			if ( shift == 0 )
				value = (unsigned char) buf[count];
			else
			{	// assume 16-bit
				i = (unsigned char) buf[count*2];
				j = (unsigned char) buf[count*2+1];

				value = j * 256 + i;
				
				if ( value > mixwaveResources->maxY() )
					value -= ( mixwaveResources->maxY() - mixwaveResources->minY() + 1);
			}
					
			value += _offset;
			value *= _scale;
				
			_out->writeF ( &value );
		}

		_dataLength -= _blockLength;
		percent = ( int ) ( ( float ) _in->filePos() / ( float ) _in->fileSize() * 100 );

		if ( _percentDone != percent )
		{
			_percentDone = percent;
			snprintf ( text, 5, "%d%%", _percentDone );
			updateMessage ( text );
		}
	}

	if ( _dataLength == 0 )
	{
		delete _in;
		delete _out;
		_done = True;
	}
}


void WavToIntCmd::setWaveDefaults ( long samplesPerSec, short bitsPerSample, short channels )
{
	_samplesPerSec = samplesPerSec;
	_bitsPerSample = bitsPerSample;
	_channels = channels;
}
